@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: about.c 9223 2009-06-08 19:47:42Z yeti-dn $
+ *  @(#) $Id: about.c 10260 2009-10-19 12:03:14Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -32,6 +32,7 @@
 #include <libgwymodule/gwymoduleloader.h>
 #include "app.h"
 #include "authors.h"
+#include "gwyddion.h"
 
 static void about_close  (void);
 static void fill_credits (GtkTextBuffer *buffer);
@@ -59,12 +60,13 @@ gwy_app_about(void)
                                         | GTK_DIALOG_DESTROY_WITH_PARENT,
                                         GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
                                         NULL);
+    g_free(s);
     gtk_dialog_set_default_response(GTK_DIALOG(about), GTK_RESPONSE_CLOSE);
     gtk_container_set_border_width(GTK_CONTAINER(about), 6);
     gtk_window_set_transient_for(GTK_WINDOW(about),
                                  GTK_WINDOW(gwy_app_main_window_get()));
     gtk_window_set_position(GTK_WINDOW(about), GTK_WIN_POS_CENTER);
-    g_free(s);
+    gwy_app_add_main_accel_group(GTK_WINDOW(about));
 
     vbox = GTK_DIALOG(about)->vbox;
     gtk_box_set_spacing(GTK_BOX(vbox), 8);
@@ -265,15 +267,18 @@ fill_features(GtkTextBuffer *buffer)
                       _("built-in SimpleFFT\n"), TRUE);
 #endif
 
+    b = FALSE;
+#if (REMOTE_BACKEND == REMOTE_NONE)
     cs = _("not available\n");
     b = TRUE;
-#ifdef HAVE_REMOTE_X11
+#elif (REMOTE_BACKEND == REMOTE_X11)
     cs = _("X11 protocol\n");
-    b = FALSE;
-#endif
-#ifdef HAVE_REMOTE_WIN32
+#elif (REMOTE_BACKEND == REMOTE_WIN32)
     cs = _("Win32 protocol\n");
-    b = FALSE;
+#elif (REMOTE_BACKEND == REMOTE_UNIQUE)
+    cs = _("LibUnique\n");
+#else
+#error "An unknown remote control backend."
 #endif
     add_credits_block(buffer, _("Remote Control"), cs, b);
 
