@@ -76,6 +76,23 @@ gwy_visibility_reset_type_get_type(void)
     return etype;
 }
 
+GType
+gwy_data_watch_event_type_get_type(void)
+{
+    static GType etype = 0;
+
+    if (etype == 0) {
+        static const GEnumValue values[] = {
+            { GWY_DATA_WATCH_EVENT_ADDED, "GWY_DATA_WATCH_EVENT_ADDED", "added" },
+            { GWY_DATA_WATCH_EVENT_CHANGED, "GWY_DATA_WATCH_EVENT_CHANGED", "changed" },
+            { GWY_DATA_WATCH_EVENT_REMOVED, "GWY_DATA_WATCH_EVENT_REMOVED", "removed" },
+            { 0, NULL, NULL }
+        };
+        etype = g_enum_register_static("GwyDataWatchEventType", values);
+    }
+    return etype;
+}
+
 #include "./menu.h"
 
 GType
