@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyexpr.c 8874 2008-12-28 12:16:22Z yeti-dn $
+ *  @(#) $Id: gwyexpr.c 11661 2010-12-03 12:32:33Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -736,7 +736,7 @@ gwy_expr_scan_tokens(GwyExpr *expr,
 
     scanner = expr->scanner;
     while ((token = g_scanner_get_next_token(scanner))) {
-        switch (token) {
+        switch ((gint)token) {
             case G_TOKEN_LEFT_PAREN:
             case G_TOKEN_RIGHT_PAREN:
             case G_TOKEN_COMMA:
@@ -762,7 +762,7 @@ gwy_expr_scan_tokens(GwyExpr *expr,
 
             default:
             g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_INVALID_TOKEN,
-                        "Invalid token");
+                        _("Invalid token"));
             gwy_expr_token_list_delete(expr, tokens);
             return FALSE;
             break;
@@ -770,7 +770,8 @@ gwy_expr_scan_tokens(GwyExpr *expr,
     }
 
     if (!tokens) {
-        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY, "No tokens");
+        g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY,
+                    _("Empty expression"));
         return FALSE;
     }
 
@@ -1012,13 +1013,13 @@ gwy_expr_transform_infix_ops(GwyExpr *expr,
         next = t->next;
         if (!next || !prev) {
             g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_MISSING_ARGUMENT,
-                        "Missing operator %c argument", operators[i]);
+                        _("Missing operator %c argument"), operators[i]);
             gwy_expr_token_list_delete(expr, tokens);
             return NULL;
         }
         if (!prev->rpn_block || !next->rpn_block) {
             g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_INVALID_ARGUMENT,
-                        "Invalid operator %c argument", operators[i]);
+                        _("Invalid operator %c argument"), operators[i]);
             gwy_expr_token_list_delete(expr, tokens);
             return NULL;
         }
@@ -1067,14 +1068,14 @@ gwy_expr_transform_functions(GwyExpr *expr,
             if (!arg) {
                 g_set_error(err, GWY_EXPR_ERROR,
                             GWY_EXPR_ERROR_MISSING_ARGUMENT,
-                            "Missing %s argument", call_table[func].name);
+                            _("Missing argument of %s"), call_table[func].name);
                 gwy_expr_token_list_delete(expr, tokens);
                 return NULL;
             }
             if (!arg->rpn_block) {
                 g_set_error(err, GWY_EXPR_ERROR,
                             GWY_EXPR_ERROR_INVALID_ARGUMENT,
-                            "Invalid %s argument", call_table[func].name);
+                            _("Invalid argument of %s"), call_table[func].name);
                 gwy_expr_token_list_delete(expr, tokens);
                 return NULL;
             }
@@ -1132,7 +1133,7 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
 
     if (tokens->token != G_TOKEN_LEFT_PAREN) {
         g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_OPENING_PARENTHESIS,
-                    "Missing opening parenthesis");
+                    _("Missing opening parenthesis"));
         goto FAIL;
     }
     tokens = gwy_expr_token_list_delete_token(expr, tokens, tokens);
@@ -1174,12 +1175,12 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     /* missing right parenthesis or empty parentheses */
     if (!t) {
         g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_CLOSING_PARENTHESIS,
-                    "Missing closing parenthesis");
+                    _("Missing closing parenthesis"));
         goto FAIL;
     }
     if (!tokens) {
         g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_EMPTY_PARENTHESES,
-                    "Empty parentheses");
+                    _("Empty parentheses"));
         goto FAIL;
     }
 
@@ -1188,7 +1189,7 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
         if (t->token == G_TOKEN_COMMA) {
             if (!t->next || !t->prev) {
                 g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_STRAY_COMMA,
-                            "Stray comma");
+                            _("Stray comma"));
                 goto FAIL;
             }
             t = t->prev;
@@ -1227,7 +1228,7 @@ gwy_expr_transform_to_rpn_real(GwyExpr *expr,
     for (t = tokens; t; t = t->next) {
         if (!t->rpn_block) {
             g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_GARBAGE,
-                        "Stray symbol %d", t->token);
+                        _("Stray symbol %d"), t->token);
             goto FAIL;
         }
     }
@@ -1279,7 +1280,7 @@ gwy_expr_transform_to_rpn(GwyExpr *expr,
 
     if (expr->tokens->next) {
         g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_GARBAGE,
-                    "Trailing garbage");
+                    _("Trailing garbage"));
         gwy_expr_token_list_delete(expr, expr->tokens);
         expr->tokens = NULL;
         return FALSE;
@@ -1299,7 +1300,7 @@ gwy_expr_transform_to_rpn(GwyExpr *expr,
 
     if (!gwy_expr_stack_check_executability(expr)) {
         g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_NOT_EXECUTABLE,
-                    "Stack not executable");
+                    _("Stack is not executable"));
         return FALSE;
     }
 
@@ -1408,7 +1409,7 @@ gwy_expr_evaluate(GwyExpr *expr,
 
     if (expr->identifiers->len > 1) {
         g_set_error(err, GWY_EXPR_ERROR, GWY_EXPR_ERROR_UNRESOLVED_IDENTIFIERS,
-                    "Unresolved identifiers");
+                    _("Unresolved identifiers"));
         return FALSE;
     }
 
