@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwyaxis.c 8995 2009-02-06 10:13:08Z yeti-dn $
+ *  @(#) $Id: gwyaxis.c 9296 2009-06-21 19:42:11Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -625,6 +625,8 @@ gwy_axis_size_request(GtkWidget *widget,
                 axis->rerequest_size = 0;
             }
         }
+        else
+            axis->rerequest_size = 0;
         gwy_debug("%p must rerequest: %d", axis, axis->rerequest_size);
         requisition->width += rect_label.width;
         break;
@@ -712,7 +714,6 @@ gwy_axis_adjust(GwyAxis *axis, gint width, gint height)
 
     if (axis->rerequest_size) {
         gwy_debug("%p issuing rerequest", axis);
-        axis->rerequest_size = 0;
         gtk_widget_queue_resize(GTK_WIDGET(axis));
     }
     if (GTK_WIDGET_DRAWABLE(axis))
@@ -1304,7 +1305,7 @@ gwy_axis_normalscale(GwyAxis *a)
 static gboolean
 gwy_axis_logscale(GwyAxis *a)
 {
-    gint i;
+    gint i, n;
     gdouble max, min, logmax, logmin, tickstep, base;
     GwyAxisLabeledTick mjt;
     GwyAxisTick mit;
@@ -1324,9 +1325,15 @@ gwy_axis_logscale(GwyAxis *a)
 
     /* Ticks will be linearly distributed again */
     /* Major ticks - will be equally ditributed in the log domain 1,10,100 */
-    tickstep = (ceil(logmax) - floor(logmin))/MAX(a->par.major_maxticks - 1, 1);
+    n = MAX(a->par.major_maxticks - 1, 1);
+    tickstep = (ceil(logmax) - floor(logmin))/n;
     tickstep = ceil(tickstep);
-    base = (ceil(logmin/tickstep) - 1)*tickstep; /* starting value */
+    for (i = 0; i < 3; i++) {
+        base = (ceil(logmin/tickstep) - 1)*tickstep; /* starting value */
+        if (base <= logmin && base + n*tickstep >= logmax)
+            break;
+        tickstep += 1.0;
+    }
     logmin = base;
     i = 0;
     do {
@@ -1404,6 +1411,11 @@ gwy_axis_scale(GwyAxis *a)
         gwy_axis_logscale(a);
     else
         gwy_axis_normalscale(a);
+
+    /* Displaying NaN or similar rubbish, just return success... */
+    if (!a->mjticks->len)
+        return 0;
+
     /*label ticks*/
     ret = gwy_axis_formatticks(a);
 
