@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: gwymoduleloader.c 10136 2009-09-17 16:40:18Z yeti-dn $
+ *  @(#) $Id: gwymoduleloader.c 11588 2010-11-19 12:38:59Z yeti-dn $
  *  Copyright (C) 2003,2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -27,8 +27,6 @@
 
 #undef GWY_MODULE_PEDANTIC_CHECK
 
-#define GWY_MODULE_QUERY_NAME G_STRINGIFY(_GWY_MODULE_QUERY)
-
 typedef struct {
     GHFunc func;
     gpointer data;
@@ -316,7 +314,7 @@ gwy_module_do_register_module(const gchar *filename,
     /* Sanity checks on the module before registration is attempted. */
     ok = TRUE;
     currenly_registered_module = modname;
-    if (!g_module_symbol(mod, GWY_MODULE_QUERY_NAME, (gpointer)&query)
+    if (!g_module_symbol(mod, "_gwy_module_query", (gpointer)&query)
         || !query) {
         g_set_error(&err, GWY_MODULE_ERROR, GWY_MODULE_ERROR_QUERY,
                     "Module contains no query function");
@@ -622,8 +620,7 @@ gwy_module_lookup(const gchar *name)
  * GWY_MODULE_QUERY:
  * @mod_info: The #GwyModuleInfo structure to return as module info.
  *
- * The declaration of module info query (the ONLY exported symbol from
- * a module).
+ * The module query must be the ONLY exported symbol from a module.
  *
  * This macro does The Right Thing necessary to export module info in a way
  * Gwyddion understands it. Put #GWY_MODULE_QUERY with the module info
@@ -631,6 +628,27 @@ gwy_module_lookup(const gchar *name)
  * semicolon after).
  **/
 
+/*
+ * GWY_MODULE_QUERY_BUILTIN:
+ * @mod_info: The #GwyModuleInfo structure to return as module info.
+ * @name: Module name.  This is the base name of the source file with dashes
+ *        and other non-identifier characters changed to underscores.  For
+ *        instance module "nt-mdt.c" should pass nt_mdt.
+ *
+ * The declaration of module info query for built-in modules.
+ *
+ * See GWY_MODULE_QUERY() for discussion.  Using this macro instead of
+ * GWY_MODULE_QUERY() changes the name of the exported query function and
+ * enables integrating the module into the gwyddion executable.  This obviously
+ * makes sense only for core modules that are distributed with Gwyddion,
+ * however, it does no harm in third party modules as this macro functions
+ * differently from GWY_MODULE_QUERY() only if preprocessor symbol
+ * %GWY_MODULE_IS_BUILTIN is defined (which you do not do in an external
+ * module).
+ *
+ * Since: 2.22
+ */
+
 /**
  * GwyModuleRegisterFunc:
  *
