@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: dataline.c 8874 2008-12-28 12:16:22Z yeti-dn $
+ *  @(#) $Id: dataline.c 11027 2010-01-27 08:50:00Z yeti-dn $
  *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -1586,6 +1586,26 @@ gwy_data_line_cumulate(GwyDataLine *data_line)
     }
 }
 
+/**
+ * gwy_data_line_sqrt:
+ * @data_line: A data line.
+ *
+ * Applies sqrt() to each element in a data line.
+ **/
+void
+gwy_data_line_sqrt(GwyDataLine *data_line)
+{
+    int i, res;
+    gdouble *data;
+    g_return_if_fail(GWY_IS_DATA_LINE(data_line));
+
+    data = data_line->data;
+    res = data_line->res;
+    for (i = 0; i < res; i++) {
+        data[i] = sqrt(data[i]);
+    }
+}
+
 
 /************************** Documentation ****************************/
 
