@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: spectra.c 10268 2009-10-21 08:56:33Z yeti-dn $
+ *  @(#) $Id: spectra.c 11316 2010-06-04 09:38:48Z yeti-dn $
  *  Copyright (C) 2006 Owain Davies, David Necas (Yeti), Petr Klapetek.
  *  E-mail: owain.davies@blueyonder.co.uk
  *          yeti@gwyddion.net, klapetek@gwyddion.net.
@@ -403,7 +403,7 @@ gwy_spectra_deserialize(const guchar *buffer,
 
     isize = 8*sizeof(guint32);
     if (2*ncurves != ncoords
-        || (nselected && (nselected + isize-1)/isize != ncurves)) {
+        || (nselected && (ncurves + isize-1)/isize != nselected)) {
         g_critical("Serialized coordinate, data and selection array size "
                    "mismatch");
         for (i = 0; i < ncurves; i++)
