@@ -1,5 +1,5 @@
 /*
- *  $Id: apefile.c 8950 2009-01-02 12:00:56Z yeti-dn $
+ *  $Id: apefile.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -20,8 +20,6 @@
  *  Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111 USA
  */
 
-/* TODO: channel types */
-
 /**
  * [FILE-MAGIC-FREEDESKTOP]
  * <mime-type type="application/x-ape-spm">
@@ -29,9 +27,27 @@
  *   <magic priority="80">
  *     <match type="string" offset="234" value="APERES"/>
  *   </magic>
+ *   <magic priority="80">
+ *     <match type="byte" offset="0" value="2">
+ *       <match type="little16" offset="3" value="7">
+ *         <match type="byte" offset="1" value="0"/>
+ *         <match type="byte" offset="1" value="1"/>
+ *         <match type="byte" offset="1" value="2"/>
+ *         <match type="byte" offset="1" value="3"/>
+ *         <match type="byte" offset="1" value="4"/>
+ *       </match>
+ *     </match>
+ *   </magic>
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * APE Research DAT
+ * .dat
+ * Read
+ **/
+
 #include <string.h>
 #include <time.h>
 #include "config.h"
@@ -56,6 +72,22 @@ typedef enum {
     SPM_MODE_LAST
 } SPMModeType;
 
+typedef enum {
+    /* They call height `Out' */
+    APE_HEIGHT   = 0,
+    APE_HEIGHT_R = 1,
+    APE_NSOM     = 2,
+    APE_NSOM_R   = 3,
+    APE_ERROR    = 4,
+    APE_ERROR_R  = 5,
+    APE_NSOM2    = 6,
+    APE_NSOM2_R  = 7,
+    APE_AUX1     = 8,
+    APE_AUX2     = 9,
+    APE_AUX1_R   = 10,
+    APE_AUX2_R   = 11,
+} APEChannel;
+
 typedef struct {
     guint version;
     SPMModeType spm_mode;
@@ -86,10 +118,28 @@ typedef struct {
     guint topo_means;
     guint optical_means;
     guint error_means;
-    guint channels;
+    guint channels;  /* bit mask, bits 0, 1, 4 and 5 should be always present */
     guint ndata;   /* computed, number of nonzero bits in channels */
     gdouble range_x;
     gdouble range_y;
+    guint subversion;
+    /* The new fields change the conversion rules, however, the simple factor
+     * above are written so that old software performs the conversion
+     * corerectly (they contain pre-multiplied or pre-divided values).  So we
+     * use that and do not care about hv_gain_z or fast2_* */
+    /* Since 2.1 */
+    gdouble hv_gain_z;
+    /* Since 2.2 */
+    gdouble fast2_0;
+    gdouble fast2_1;
+    gdouble fast2_2;
+    gdouble fast2_3;
+    /* Since 2.3 */
+    gboolean pg850_image;
+    /* Since 2.4 */
+    gint xy_hv_status;
+    gint z_hv_status;
+    /* The data */
     GwyDataField **data;
 } APEFile;
 
@@ -117,7 +167,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Imports APE (Applied Physics and Engineering) data files."),
     "Yeti <yeti@gwyddion.net>",
-    "0.8",
+    "0.9",
     "David Nečas (Yeti) & Petr Klapetek",
     "2005",
 };
@@ -179,7 +229,7 @@ apefile_load(const gchar *filename,
     gsize size = 0;
     GError *err = NULL;
     GwyDataField *dfield = NULL;
-    guint b, n;
+    guint b, i, n;
 
     if (!gwy_file_get_contents(filename, &buffer, &size, &err)) {
         err_GET_FILE_CONTENTS(error, &err);
@@ -241,6 +291,24 @@ apefile_load(const gchar *filename,
         apefile.ndata += (b & 1);
     apefile.range_x = gwy_get_gfloat_le(&p);
     apefile.range_y = gwy_get_gfloat_le(&p);
+    apefile.subversion = gwy_get_guint16_le(&p);
+    /* Read everything since the header is long enough, check the version
+     * later when we decide whether to use these values or not. */
+    /* Since 2.1 */
+    apefile.hv_gain_z = gwy_get_gfloat_le(&p);
+    /* Since 2.2 */
+    apefile.fast2_0 = gwy_get_gdouble_le(&p);
+    apefile.fast2_1 = gwy_get_gdouble_le(&p);
+    apefile.fast2_2 = gwy_get_gdouble_le(&p);
+    apefile.fast2_3 = gwy_get_gdouble_le(&p);
+    /* Since 2.3 */
+    apefile.pg850_image = !!gwy_get_guint16_le(&p);
+    /* Since 2.4 */
+    apefile.xy_hv_status = gwy_get_gint16_le(&p);
+    apefile.z_hv_status = gwy_get_gint16_le(&p);
+    /* reserved */
+    p += 2;
+
     apefile.xreal = apefile.maxr_x * apefile.x_piezo_factor * apefile.range_x
                     * apefile.hv_gain/65535.0 * 1e-9;
     apefile.yreal = apefile.maxr_y * apefile.y_piezo_factor * apefile.range_y
@@ -254,10 +322,9 @@ apefile_load(const gchar *filename,
         g_warning("Real y size is 0.0, fixing to 1.0");
         apefile.yreal = 1.0;
     }
-    /* reserved */
-    p += 46;
 
-    gwy_debug("version = %u, spm_mode = %u", apefile.version, apefile.spm_mode);
+    gwy_debug("version = %u.%u, spm_mode = %u",
+              apefile.version, apefile.subversion, apefile.spm_mode);
     gwy_debug("scan_date = %f", apefile.scan_date);
     gwy_debug("maxr_x = %g, maxr_y = %g", apefile.maxr_x, apefile.maxr_y);
     gwy_debug("x_offset = %u, y_offset = %u",
@@ -298,19 +365,43 @@ apefile_load(const gchar *filename,
     container = gwy_container_new();
     /* All metadata seems to be per-file (global) */
     meta = apefile_get_metadata(&apefile);
-    for (n = 0; n < apefile.ndata; n++) {
+    for (b = apefile.channels, n = 0, i = 0; b; b = b >> 1, i++) {
         GwyContainer *tmp;
-        gchar key[24];
+        gchar *title;
+        gchar key[32];
+
+        if (!(b & 1))
+            continue;
 
         g_snprintf(key, sizeof(key), "/%d/data", n);
         dfield = apefile.data[n];
         gwy_container_set_object_by_name(container, key, dfield);
         g_object_unref(apefile.data[n]);
 
+        g_snprintf(key, sizeof(key), "/%d/data/title", n);
+        title = gwy_enuml_to_string(i,
+                                    "Height",   APE_HEIGHT,
+                                    "Height-R", APE_HEIGHT_R,
+                                    "NSOM",     APE_NSOM,
+                                    "NSOM-R",   APE_NSOM_R,
+                                    "Error",    APE_ERROR,
+                                    "Error-R",  APE_ERROR_R,
+                                    "NSOM2",    APE_NSOM2,
+                                    "NSOM2-R",  APE_NSOM2_R,
+                                    "Aux1",     APE_AUX1,
+                                    "Aux2",     APE_AUX2,
+                                    "Aux1-R",   APE_AUX1_R,
+                                    "Aux2-R",   APE_AUX2_R,
+                                    NULL);
+        if (title && *title)
+            gwy_container_set_string_by_name(container, key, g_strdup(title));
+
         tmp = gwy_container_duplicate(meta);
         g_snprintf(key, sizeof(key), "/%d/meta", n);
         gwy_container_set_object_by_name(container, key, tmp);
         g_object_unref(tmp);
+
+        n++;
     }
 
     g_object_unref(meta);
@@ -326,27 +417,55 @@ fill_data_fields(APEFile *apefile,
     GwyDataField *dfield;
     GwySIUnit *unit;
     gdouble *data;
-    guint n, i, j;
+    const gchar *zunit;
+    guint b, n, i, j, k;
+    gdouble q;
 
     apefile->data = g_new0(GwyDataField*, apefile->ndata);
-    for (n = 0; n < apefile->ndata; n++) {
+    for (b = apefile->channels, n = 0, k = 0; b; b = b >> 1, k++) {
+        if (!(b & 1))
+            continue;
+
         dfield = gwy_data_field_new(apefile->res, apefile->res,
                                     apefile->xreal, apefile->yreal, FALSE);
         unit = gwy_data_field_get_si_unit_xy(dfield);
         gwy_si_unit_set_from_string(unit, "m");
-        unit = gwy_data_field_get_si_unit_z(dfield);
-        gwy_si_unit_set_from_string(unit, "m");
 
         data = gwy_data_field_get_data(dfield);
         buffer += (apefile->res + 1)*sizeof(float);
+        switch (k) {
+            case APE_HEIGHT:
+            case APE_HEIGHT_R:
+            case APE_AUX2:
+            case APE_AUX2_R:
+            q = apefile->z_piezo_factor * 1e-9;
+            zunit = "m";
+            break;
+
+            case APE_AUX1:
+            case APE_AUX1_R:
+            q = 1.0;
+            zunit = apefile->pg850_image ? "A" : "V";
+            break;
+
+            default:
+            q = 1.0;
+            zunit = "V";
+            break;
+        }
+
+        unit = gwy_data_field_get_si_unit_z(dfield);
+        gwy_si_unit_set_from_string(unit, zunit);
+
         for (i = 0; i < apefile->res; i++) {
+            /* There is always one ignored record, do not ask me why... */
             buffer += sizeof(float);
             for (j = 0; j < apefile->res; j++) {
-                *(data++) = gwy_get_gfloat_le(&buffer);
+                *(data++) = q*gwy_get_gfloat_le(&buffer);
             }
         }
         apefile->data[n] = dfield;
-        gwy_data_field_multiply(dfield, apefile->z_piezo_factor * 1e-9);
+        n++;
     }
 }
 
@@ -625,7 +744,7 @@ format_vt_date(gdouble vt_date)
     if (!gwy_VarUdateFromDate(vt_date, &udate))
         return NULL;
 
-    memset(&tm, 0, sizeof(tm));
+    gwy_clear(&tm, 1);
     gwy_debug("Date: %d-%d-%d %d:%d:%d",
               udate.st.wYear, udate.st.wMonth, udate.st.wDay,
               udate.st.wHour, udate.st.wMinute, udate.st.wSecond);
