@@ -1,5 +1,5 @@
 /*
- *  $Id: igorfile.c 11383 2010-09-07 09:31:58Z yeti-dn $
+ *  $Id: igorfile.c 11623 2010-11-29 07:52:02Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -714,7 +714,7 @@ igor_read_data_field(const IgorFile *igorfile,
                                       &power10);
     q = pow10(power10);
 
-    switch (wave5->type) {
+    switch ((guint)wave5->type) {
         case IGOR_INT8:
         {
             const gint8 *ps = (const gint8*)buffer;
