@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: jpkscan.c 10195 2009-10-01 23:04:26Z yeti-dn $
+ *  @(#) $Id: jpkscan.c 11325 2010-06-15 06:54:23Z yeti-dn $
  *  Loader for JPK Image Scans.
  *  Copyright (C) 2005  JPK Instruments AG.
  *  Written by Sven Neumann <neumann@jpk.com>.
@@ -206,7 +206,7 @@ jpkscan_load(const gchar *filename,
         switch (photo) {
             case GWY_TIFF_PHOTOMETRIC_MIN_IS_BLACK:
             case GWY_TIFF_PHOTOMETRIC_MIN_IS_WHITE:
-            if (reader->bits_per_sample == 16)
+            if (reader->bits_per_sample == 16 || reader->bits_per_sample == 32)
                 break;
 
             default:
