@@ -1,5 +1,5 @@
 /*
- *  $Id: sdfile.c 9037 2009-03-07 16:45:28Z yeti-dn $
+ *  $Id: sdfile.c 10195 2009-10-01 23:04:26Z yeti-dn $
  *  Copyright (C) 2005 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -36,6 +36,20 @@
  * </mime-type>
  **/
 
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Surfstand Surface Data File
+ * .sdf
+ * Read Export
+ **/
+
+/**
+ * [FILE-MAGIC-USERGUIDE]
+ * Micromap SDFA
+ * .sdfa
+ * Read
+ **/
+
 #include "config.h"
 #include <string.h>
 #include <stdlib.h>
@@ -537,7 +551,7 @@ sdfile_read_header_bin(const guchar **p,
         return FALSE;
     }
 
-    memset(sdfile, 0, sizeof(SDFile));
+    gwy_clear(sdfile, 1);
     get_CHARARRAY(sdfile->version, p);
     get_CHARARRAY(sdfile->manufacturer, p);
     get_CHARARRAY(sdfile->creation, p);
@@ -610,7 +624,7 @@ sdfile_read_header_text(gchar **buffer,
         return FALSE;
     }
 
-    memset(sdfile, 0, sizeof(SDFile));
+    gwy_clear(sdfile, 1);
     p = *buffer;
 
     val = g_strstrip(gwy_str_next_line(&p));
