@@ -420,14 +420,14 @@ fit_dialog(FitArgs *args)
     label = gtk_label_new(_("Instant:"));
     gtk_box_pack_start(GTK_BOX(hbox2), label, FALSE, FALSE, 0);
 
-    controls.auto_estimate = gtk_check_button_new_with_mnemonic("e_stimate");
+    controls.auto_estimate = gtk_check_button_new_with_mnemonic(_("e_stimate"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls.auto_estimate),
                                  args->auto_estimate);
     gtk_box_pack_start(GTK_BOX(hbox2), controls.auto_estimate, FALSE, FALSE, 0);
     g_signal_connect(controls.auto_estimate, "toggled",
                      G_CALLBACK(auto_estimate_changed), &controls);
 
-    controls.auto_plot = gtk_check_button_new_with_mnemonic("p_lot");
+    controls.auto_plot = gtk_check_button_new_with_mnemonic(_("p_lot"));
     gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(controls.auto_plot),
                                  args->auto_plot);
     gtk_box_pack_start(GTK_BOX(hbox2), controls.auto_plot, FALSE, FALSE, 0);
@@ -1172,6 +1172,20 @@ copy_param(GObject *button,
     gtk_widget_activate(cntrl->init);
 }
 
+static void
+render_translated_name(G_GNUC_UNUSED GtkCellLayout *layout,
+                       GtkCellRenderer *renderer,
+                       GtkTreeModel *model,
+                       GtkTreeIter *iter,
+                       gpointer data)
+{
+    guint i = GPOINTER_TO_UINT(data);
+    const gchar *text;
+
+    gtk_tree_model_get(model, iter, i, &text, -1);
+    g_object_set(renderer, "text", _(text), NULL);
+}
+
 static GtkWidget*
 function_selector_new(GCallback callback,
                       gpointer cbdata,
@@ -1180,7 +1194,7 @@ function_selector_new(GCallback callback,
     GtkCellRenderer *renderer;
     GtkWidget *combo;
     GwyInventoryStore *store;
-    gint i;
+    guint i;
 
     store = gwy_inventory_store_new(gwy_fd_curve_presets());
 
@@ -1190,7 +1204,9 @@ function_selector_new(GCallback callback,
     renderer = gtk_cell_renderer_text_new();
     gtk_cell_layout_pack_start(GTK_CELL_LAYOUT(combo), renderer, TRUE);
     i = gwy_inventory_store_get_column_by_name(store, "name");
-    gtk_cell_layout_add_attribute(GTK_CELL_LAYOUT(combo), renderer, "text", i);
+    gtk_cell_layout_set_cell_data_func(GTK_CELL_LAYOUT(combo), renderer,
+                                       render_translated_name,
+                                       GUINT_TO_POINTER(i), NULL);
     gtk_combo_box_set_active(GTK_COMBO_BOX(combo), current);
     g_signal_connect(combo, "changed", callback, cbdata);
 
