@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: fit_2d.c 8910 2008-12-30 12:41:01Z yeti-dn $
+ *  @(#) $Id: fit_2d.c 11549 2010-11-15 10:16:35Z yeti-dn $
  *  Copyright (C) 2004 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -879,7 +879,7 @@ static void
 fit_2d_sanitize_args(Fit2DArgs *args)
 {
     args->display_type = MIN(args->display_type, GWY_FIT_2D_DISPLAY_DIFF);
-    args->function_type = MIN(args->display_type, GWY_FIT_2D_FIT_SPHERE_DOWN);
+    args->function_type = MIN(args->function_type, GWY_FIT_2D_FIT_SPHERE_DOWN);
 }
 
 static void
