@@ -1,7 +1,7 @@
 /*
- *  @(#) $Id: linecorrect.c 7511 2007-02-11 18:49:21Z yeti-dn $
- *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek.
- *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
+ *  @(#) $Id: linecorrect.c 10404 2009-11-11 08:17:26Z yeti-dn $
+ *  Copyright (C) 2003 David Necas (Yeti), Petr Klapetek, Luke Somers.
+ *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net, lsomers@sas.upenn.edu.
  *
  *  This program is free software; you can redistribute it and/or modify
  *  it under the terms of the GNU General Public License as published by
@@ -43,6 +43,8 @@ static void     line_correct_modus                 (GwyContainer *data,
                                                     GwyRunType run);
 static void     line_correct_median                (GwyContainer *data,
                                                     GwyRunType run);
+static void     line_correct_median_difference     (GwyContainer *data,
+                                                    GwyRunType run);
 static void     line_correct_match                 (GwyContainer *data,
                                                     GwyRunType run);
 static void     line_correct_step                  (GwyContainer *data,
@@ -60,9 +62,9 @@ static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
     &module_register,
     N_("Corrects line defects (mostly experimental algorithms)."),
-    "Yeti <yeti@gwyddion.net>",
-    "1.6",
-    "David Nečas (Yeti) & Petr Klapetek",
+    "Yeti <yeti@gwyddion.net>, Luke Somers <lsomers@sas.upenn.edu>",
+    "1.7",
+    "David Nečas (Yeti) & Petr Klapetek & Luke Somers",
     "2004",
 };
 
@@ -85,6 +87,14 @@ module_register(void)
                               LINECORR_RUN_MODES,
                               GWY_MENU_FLAG_DATA,
                               N_("Correct lines by matching height median"));
+    gwy_process_func_register("line_correct_median_difference",
+                              (GwyProcessFunc)&line_correct_median_difference,
+                              N_("/_Correct Data/Median Di_fference Line Correction"),
+                              GWY_STOCK_LINE_LEVEL,
+                              LINECORR_RUN_MODES,
+                              GWY_MENU_FLAG_DATA,
+                              N_("Correct lines by zeroing median "
+                                 "of differences between neighbor lines"));
     gwy_process_func_register("line_correct_match",
                               (GwyProcessFunc)&line_correct_match,
                               N_("/_Correct Data/Ma_tch Line Correction"),
@@ -164,7 +174,7 @@ line_correct_median(GwyContainer *data, GwyRunType run)
 
     for (i = 0; i < yres; i++) {
         gwy_data_field_get_row(dfield, line, i);
-        median = gwy_data_line_get_median(line);
+        median = gwy_math_median(xres, gwy_data_line_get_data(line));
         gwy_data_line_set_val(modi, i, median);
     }
     median = gwy_data_line_get_median(modi);
@@ -179,6 +189,41 @@ line_correct_median(GwyContainer *data, GwyRunType run)
     gwy_data_field_data_changed(dfield);
 }
 
+static void
+line_correct_median_difference(GwyContainer *data, GwyRunType run)
+{
+    GwyDataField *dfield;
+    gdouble *diffs, *row, *prev;
+    gint xres, yres, i, j;
+    gdouble median;
+    GQuark dquark;
+
+    g_return_if_fail(run & GWY_RUN_IMMEDIATE);
+    gwy_app_data_browser_get_current(GWY_APP_DATA_FIELD, &dfield,
+                                     GWY_APP_DATA_FIELD_KEY, &dquark,
+                                     0);
+    g_return_if_fail(dfield && dquark);
+    gwy_app_undo_qcheckpointv(data, 1, &dquark);
+
+    xres = gwy_data_field_get_xres(dfield);
+    yres = gwy_data_field_get_yres(dfield);
+    row = gwy_data_field_get_data(dfield);
+    diffs = g_new(gdouble, xres);
+
+    for (i = 1; i < yres; i++) {
+        prev = row;
+        row += xres;
+        for (j = 0; j < xres; j++)
+            diffs[j] = prev[j] - row[j];
+        median = gwy_math_median(xres, diffs);
+        for (j = 0; j < xres; j++)
+            row[j] += median;
+    }
+
+    g_free(diffs);
+    gwy_data_field_data_changed(dfield);
+}
+
 static gdouble
 find_minima_golden(gdouble (*func)(gdouble x, gpointer data),
                    gdouble from,
