@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: mark_with.c 9066 2009-03-14 21:12:23Z yeti-dn $
+ *  @(#) $Id: mark_with.c 10296 2009-10-26 19:13:37Z yeti-dn $
  *  Copyright (C) 2009 David Necas (Yeti).
  *  E-mail: yeti@gwyddion.net.
  *
@@ -151,7 +151,7 @@ static GwyModuleInfo module_info = {
     &module_register,
     N_("Creates or modifies a mask using other channels."),
     "Yeti <yeti@gwyddion.net>",
-    "1.0",
+    "1.1",
     "David Nečas (Yeti)",
     "2009",
 };
@@ -196,7 +196,7 @@ mark_dialog(MarkArgs *args,
             const GwyDataObjectId *target,
             GQuark mquark)
 {
-    GtkWidget *dialog, *hbox, *vbox, *label;
+    GtkWidget *dialog, *hbox, *vbox, *label, *vbox2;
     GtkTable *table;
     GtkTooltips *tips;
     MarkControls controls;
@@ -241,12 +241,27 @@ mark_dialog(MarkArgs *args,
     gtk_box_pack_start(GTK_BOX(hbox), vbox, FALSE, FALSE, 0);
 
     /* Source mask preview */
+    /* Silly, boxes do not have spacing *between* items... */
+    vbox2 = gtk_vbox_new(FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);
+
     controls.view_source = gwy_data_view_new(controls.mydata);
-    gtk_box_pack_start(GTK_BOX(vbox), controls.view_source, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox2), controls.view_source, FALSE, FALSE, 0);
+
+    label = gtk_label_new(_("Operand"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
     /* Result preview */
+    vbox2 = gtk_vbox_new(FALSE, 2);
+    gtk_box_pack_start(GTK_BOX(vbox), vbox2, FALSE, FALSE, 0);
+
     controls.view_result = gwy_data_view_new(controls.mydata);
-    gtk_box_pack_start(GTK_BOX(vbox), controls.view_result, FALSE, FALSE, 0);
+    gtk_box_pack_start(GTK_BOX(vbox2), controls.view_result, FALSE, FALSE, 0);
+
+    label = gtk_label_new(_("Result"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_box_pack_start(GTK_BOX(vbox2), label, FALSE, FALSE, 0);
 
     /* Controls */
     table = GTK_TABLE(gtk_table_new(13, 4, FALSE));
@@ -522,14 +537,18 @@ static void
 mark_with_changed(GtkWidget *button,
                   MarkControls *controls)
 {
-    controls->args->mark_with = gwy_radio_button_get_value(button);
+    MarkArgs *args = controls->args;
 
+    if (!gtk_toggle_button_get_active(GTK_TOGGLE_BUTTON(button)))
+        return;
+
+    args->mark_with = gwy_radio_button_get_value(button);
     gwy_table_hscale_set_sensitive(GTK_OBJECT(controls->min),
-                                   controls->args->mark_with != MARK_WITH_MASK);
+                                   args->mark_with != MARK_WITH_MASK);
     gwy_table_hscale_set_sensitive(GTK_OBJECT(controls->max),
-                                   controls->args->mark_with != MARK_WITH_MASK);
+                                   args->mark_with != MARK_WITH_MASK);
 
-    channel_changed(GWY_DATA_CHOOSER(controls->channels[controls->args->mark_with]),
+    channel_changed(GWY_DATA_CHOOSER(controls->channels[args->mark_with]),
                     controls);
 }
 
@@ -547,6 +566,7 @@ channel_changed(GwyDataChooser *chooser,
     if (controls->args->update) {
         setup_source_view_data(controls);
         update_source_mask(controls);
+        perform_operation(controls);
     }
 }
 
