@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: maskedit.c 11385 2010-09-10 12:51:24Z yeti-dn $
+ *  @(#) $Id: maskedit.c 11627 2010-11-29 08:29:28Z yeti-dn $
  *  Copyright (C) 2003-2006 David Necas (Yeti), Petr Klapetek.
  *  E-mail: yeti@gwyddion.net, klapetek@gwyddion.net.
  *
@@ -39,33 +39,51 @@
 #define GWY_TOOL_MASK_EDITOR_GET_CLASS(obj)  (G_TYPE_INSTANCE_GET_CLASS((obj), GWY_TYPE_TOOL_MASK_EDITOR, GwyToolMaskEditorClass))
 
 enum {
-    SENS_DATA = 1 << 0,
-    SENS_MASK = 1 << 1,
+    SENS_DATA   = 1 << 0,
+    SENS_MASK   = 1 << 1,
 };
 
+typedef enum {
+    MASK_EDIT_STYLE_SHAPES  = 0,
+    MASK_EDIT_STYLE_DRAWING = 1,
+    MASK_NSTYLES
+} MaskEditStyle;
+
 typedef enum {
     MASK_EDIT_SET       = 0,
     MASK_EDIT_ADD       = 1,
     MASK_EDIT_REMOVE    = 2,
-    MASK_EDIT_INTERSECT = 3
+    MASK_EDIT_INTERSECT = 3,
+    MASK_NMODES
 } MaskEditMode;
 
 typedef enum {
-    MASK_SHAPE_RECTANGLE = 0,
-    MASK_SHAPE_ELLIPSE   = 1,
-    MASK_SHAPE_LINE      = 2,
+    MASK_SHAPE_RECTANGLE  = 0,
+    MASK_SHAPE_ELLIPSE    = 1,
+    MASK_SHAPE_LINE       = 2,
     MASK_NSHAPES
 } MaskEditShape;
 
+typedef enum {
+    MASK_TOOL_PAINT_DRAW  = 0,
+    MASK_TOOL_PAINT_ERASE = 1,
+    MASK_TOOL_FILL_DRAW   = 2,
+    MASK_TOOL_FILL_ERASE  = 3,
+    MASK_NTOOLS
+} MaskEditTool;
+
 typedef struct _GwyToolMaskEditor      GwyToolMaskEditor;
 typedef struct _GwyToolMaskEditorClass GwyToolMaskEditorClass;
 
 typedef void (*FieldFillFunc)(GwyDataField*, gint, gint, gint, gint, gdouble);
 
 typedef struct {
+    MaskEditStyle style;
     MaskEditMode mode;
     MaskEditShape shape;
+    MaskEditTool tool;
     gint32 gsamount;
+    gint32 radius;
     gboolean from_border;
     gboolean prevent_merge;
 } ToolArgs;
@@ -76,15 +94,23 @@ struct _GwyToolMaskEditor {
     ToolArgs args;
 
     GwySensitivityGroup *sensgroup;
+    GSList *style;
     GSList *mode;
     GSList *shape;
+    GSList *tool;
 
+    GtkObject *radius;
     GtkObject *gsamount;
     GtkWidget *from_border;
     GtkWidget *prevent_merge;
 
+    /* paintrbrush only */
+    gboolean drawing_started;
+    gint oldisel[2];
+
     /* potential class data */
     GType layer_types[MASK_NSHAPES];
+    GType layer_type_point;
 };
 
 struct _GwyToolMaskEditorClass {
@@ -99,9 +125,13 @@ static void  gwy_tool_mask_editor_init_dialog        (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_data_switched      (GwyTool *gwytool,
                                                       GwyDataView *data_view);
 static void  gwy_tool_mask_editor_mask_changed       (GwyPlainTool *plain_tool);
+static void  gwy_tool_mask_editor_style_changed      (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_mode_changed       (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_shape_changed      (GwyToolMaskEditor *tool);
+static void  gwy_tool_mask_editor_tool_changed       (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_setup_layer        (GwyToolMaskEditor *tool);
+static void  gwy_tool_mask_editor_radius_changed     (GtkAdjustment *adj,
+                                                      GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_gsamount_changed   (GtkAdjustment *adj,
                                                       GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_from_border_changed(GtkToggleButton *toggle,
@@ -115,6 +145,11 @@ static void  gwy_tool_mask_editor_grow               (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_shrink             (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_fill_voids         (GwyToolMaskEditor *tool);
 static void  gwy_tool_mask_editor_selection_finished (GwyPlainTool *plain_tool);
+static void  gwy_tool_mask_editor_bucket_fill        (GwyToolMaskEditor *tool,
+                                                      gint j,
+                                                      gint i);
+static void  gwy_tool_mask_editor_selection_changed  (GwyPlainTool *plain_tool,
+                                                      gint hint);
 
 static GwyModuleInfo module_info = {
     GWY_MODULE_ABI_VERSION,
@@ -122,24 +157,30 @@ static GwyModuleInfo module_info = {
     N_("Mask editor tool, allows to interactively add or remove parts "
        "of mask."),
     "Yeti <yeti@gwyddion.net>",
-    "2.9",
+    "3.0",
     "David Nečas (Yeti) & Petr Klapetek",
     "2004",
 };
 
 static const gchar *const shape_selection_names[MASK_NSHAPES] = {
-    "rectangle", "ellipse", "line",
+    "rectangle", "ellipse", "line"
 };
 
+static const gchar style_key[]         = "/module/maskeditor/style";
 static const gchar mode_key[]          = "/module/maskeditor/mode";
 static const gchar shape_key[]         = "/module/maskeditor/shape";
+static const gchar tool_key[]          = "/module/maskeditor/tool";
+static const gchar radius_key[]        = "/module/maskeditor/radius";
 static const gchar gsamount_key[]      = "/module/maskeditor/gsamount";
 static const gchar from_border_key[]   = "/module/maskeditor/from_border";
 static const gchar prevent_merge_key[] = "/module/maskeditor/prevent_merge";
 
 static const ToolArgs default_args = {
+    MASK_EDIT_STYLE_SHAPES,
     MASK_EDIT_SET,
     MASK_SHAPE_RECTANGLE,
+    MASK_TOOL_PAINT_DRAW,
+    5,
     1,
     FALSE,
     TRUE,
@@ -173,6 +214,7 @@ gwy_tool_mask_editor_class_init(GwyToolMaskEditorClass *klass)
     tool_class->data_switched = gwy_tool_mask_editor_data_switched;
 
     ptool_class->mask_changed = gwy_tool_mask_editor_mask_changed;
+    ptool_class->selection_changed = gwy_tool_mask_editor_selection_changed;
     ptool_class->selection_finished = gwy_tool_mask_editor_selection_finished;
 }
 
@@ -185,10 +227,16 @@ gwy_tool_mask_editor_finalize(GObject *object)
     tool = GWY_TOOL_MASK_EDITOR(object);
 
     settings = gwy_app_settings_get();
+    gwy_container_set_enum_by_name(settings, style_key,
+                                   tool->args.style);
     gwy_container_set_enum_by_name(settings, mode_key,
                                    tool->args.mode);
     gwy_container_set_enum_by_name(settings, shape_key,
                                    tool->args.shape);
+    gwy_container_set_enum_by_name(settings, tool_key,
+                                   tool->args.tool);
+    gwy_container_set_int32_by_name(settings, radius_key,
+                                    tool->args.radius);
     gwy_container_set_int32_by_name(settings, gsamount_key,
                                     tool->args.gsamount);
     gwy_container_set_boolean_by_name(settings, from_border_key,
@@ -218,13 +266,23 @@ gwy_tool_mask_editor_init(GwyToolMaskEditor *tool)
         if (!tool->layer_types[i])
             return;
     }
+    tool->layer_type_point = gwy_plain_tool_check_layer_type(plain_tool,
+                                                             "GwyLayerPoint");
+    if (!tool->layer_type_point)
+        return;
 
     settings = gwy_app_settings_get();
     tool->args = default_args;
+    gwy_container_gis_enum_by_name(settings, style_key,
+                                   &tool->args.style);
     gwy_container_gis_enum_by_name(settings, mode_key,
                                    &tool->args.mode);
     gwy_container_gis_enum_by_name(settings, shape_key,
                                    &tool->args.shape);
+    gwy_container_gis_enum_by_name(settings, tool_key,
+                                   &tool->args.tool);
+    gwy_container_gis_int32_by_name(settings, radius_key,
+                                    &tool->args.radius);
     gwy_container_gis_int32_by_name(settings, gsamount_key,
                                     &tool->args.gsamount);
     gwy_container_gis_boolean_by_name(settings, from_border_key,
@@ -232,10 +290,18 @@ gwy_tool_mask_editor_init(GwyToolMaskEditor *tool)
     gwy_container_gis_boolean_by_name(settings, prevent_merge_key,
                                       &tool->args.prevent_merge);
 
+    tool->args.style = MIN(tool->args.style, MASK_NSTYLES-1);
+    tool->args.mode = MIN(tool->args.mode, MASK_NMODES-1);
     tool->args.shape = MIN(tool->args.shape, MASK_NSHAPES-1);
-    gwy_plain_tool_connect_selection(plain_tool,
-                                     tool->layer_types[tool->args.shape],
-                                     shape_selection_names[tool->args.shape]);
+    tool->args.tool = MIN(tool->args.tool, MASK_NTOOLS-1);
+
+    if (tool->args.style == MASK_EDIT_STYLE_SHAPES)
+        gwy_plain_tool_connect_selection(plain_tool,
+                                         tool->layer_types[tool->args.shape],
+                                         shape_selection_names[tool->args.shape]);
+    else
+        gwy_plain_tool_connect_selection(plain_tool,
+                                         tool->layer_type_point, "pointer");
 
     gwy_tool_mask_editor_init_dialog(tool);
 }
@@ -286,6 +352,28 @@ gwy_tool_mask_editor_init_dialog(GwyToolMaskEditor *tool)
             GWY_STOCK_MASK_LINE,
             N_("Thin lines"),
         },
+    },
+    tools[] = {
+        {
+            MASK_TOOL_PAINT_DRAW,
+            GWY_STOCK_MASK_PAINT_DRAW,
+            N_("Freehand mask drawing"),
+        },
+        {
+            MASK_TOOL_PAINT_ERASE,
+            GWY_STOCK_MASK_PAINT_ERASE,
+            N_("Freehand mask erasing"),
+        },
+        {
+            MASK_TOOL_FILL_DRAW,
+            GWY_STOCK_MASK_FILL_DRAW,
+            N_("Fill continous empty areas with mask"),
+        },
+        {
+            MASK_TOOL_FILL_ERASE,
+            GWY_STOCK_MASK_FILL_ERASE,
+            N_("Erase continous parts of mask"),
+        },
     };
 
     GtkRadioButton *group;
@@ -302,7 +390,7 @@ gwy_tool_mask_editor_init_dialog(GwyToolMaskEditor *tool)
     sizegroup = gtk_size_group_new(GTK_SIZE_GROUP_HORIZONTAL);
     tool->sensgroup = gwy_sensitivity_group_new();
 
-    table = GTK_TABLE(gtk_table_new(10, 4, FALSE));
+    table = GTK_TABLE(gtk_table_new(13, 4, FALSE));
     gtk_table_set_col_spacings(table, 6);
     gtk_table_set_row_spacings(table, 2);
     gtk_container_set_border_width(GTK_CONTAINER(table), 4);
@@ -315,6 +403,16 @@ gwy_tool_mask_editor_init_dialog(GwyToolMaskEditor *tool)
     gtk_table_attach(table, label, 0, 4, row, row+1, GTK_FILL, 0, 0, 0);
     row++;
 
+    /* Shapes */
+    button = gtk_radio_button_new_with_mnemonic(NULL, _("_Shapes"));
+    tool->style = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
+    gwy_radio_button_set_value(button, MASK_EDIT_STYLE_SHAPES);
+    gtk_table_attach(table, button, 0, 3, row, row+1, GTK_FILL, 0, 0, 0);
+    g_signal_connect_swapped(button, "clicked",
+                             G_CALLBACK(gwy_tool_mask_editor_style_changed),
+                             tool);
+    row++;
+
     /* Mode */
     label = gtk_label_new(_("Mode:"));
     gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
@@ -371,10 +469,60 @@ gwy_tool_mask_editor_init_dialog(GwyToolMaskEditor *tool)
     }
     tool->shape = gtk_radio_button_get_group(group);
     gwy_radio_buttons_set_current(tool->shape, tool->args.shape);
-    gtk_table_set_row_spacing(table, row, 12);
+    row++;
+
+    /* Drawing Tools */
+    gtk_table_set_row_spacing(table, row-1, 8);
+    button = gtk_radio_button_new_with_mnemonic(tool->style,
+                                                _("_Drawing Tools"));
+    tool->style = gtk_radio_button_get_group(GTK_RADIO_BUTTON(button));
+    gwy_radio_button_set_value(button, MASK_EDIT_STYLE_DRAWING);
+    gtk_table_attach(table, button, 0, 3, row, row+1, GTK_FILL, 0, 0, 0);
+    g_signal_connect_swapped(button, "clicked",
+                             G_CALLBACK(gwy_tool_mask_editor_style_changed),
+                             tool);
+    row++;
+
+    /* Tool */
+    label = gtk_label_new(_("Tool:"));
+    gtk_misc_set_alignment(GTK_MISC(label), 0.0, 0.5);
+    gtk_table_attach(table, label, 0, 1, row, row+1, GTK_FILL, 0, 0, 0);
+
+    hbox = GTK_BOX(gtk_hbox_new(FALSE, 0));
+    gtk_table_attach(table, GTK_WIDGET(hbox),
+                     1, 4, row, row+1, GTK_EXPAND | GTK_FILL, 0, 0, 0);
+
+    group = NULL;
+    for (i = 0; i < G_N_ELEMENTS(tools); i++) {
+        button = gtk_radio_button_new_from_widget(group);
+        g_object_set(button, "draw-indicator", FALSE, NULL);
+        image = gtk_image_new_from_stock(tools[i].stock_id,
+                                         GTK_ICON_SIZE_LARGE_TOOLBAR);
+        gtk_container_add(GTK_CONTAINER(button), image);
+        gwy_radio_button_set_value(button, tools[i].type);
+        gtk_box_pack_start(hbox, button, FALSE, FALSE, 0);
+        gtk_tooltips_set_tip(tips, button, gettext(tools[i].text), NULL);
+        g_signal_connect_swapped(button, "clicked",
+                                 G_CALLBACK(gwy_tool_mask_editor_tool_changed),
+                                 tool);
+        if (!group)
+            group = GTK_RADIO_BUTTON(button);
+    }
+    tool->tool = gtk_radio_button_get_group(group);
+    gwy_radio_buttons_set_current(tool->tool, tool->args.tool);
+    row++;
+
+    /* Radius */
+    tool->radius = gtk_adjustment_new(tool->args.radius, 1.0, 15.0,
+                                      1.0, 1.0, 0.0);
+    gwy_table_attach_hscale(GTK_WIDGET(table), row, _("_Radius:"), "px",
+                            tool->radius, 0);
+    g_signal_connect(tool->radius, "value-changed",
+                     G_CALLBACK(gwy_tool_mask_editor_radius_changed), tool);
     row++;
 
     /* Actions */
+    gtk_table_set_row_spacing(table, row-1, 8);
     label = gwy_label_new_header(_("Actions"));
     gtk_table_attach(table, label, 0, 4, row, row+1, GTK_FILL, 0, 0, 0);
     row++;
@@ -416,7 +564,7 @@ gwy_tool_mask_editor_init_dialog(GwyToolMaskEditor *tool)
 
     label = gtk_label_new(NULL);
     gtk_box_pack_start(hbox, label, TRUE, TRUE, 0);
-    gtk_table_set_row_spacing(table, row, 12);
+    gtk_table_set_row_spacing(table, row, 8);
     row++;
 
     /* Grow/Shrink */
@@ -482,6 +630,7 @@ gwy_tool_mask_editor_init_dialog(GwyToolMaskEditor *tool)
     row++;
 
     gwy_tool_add_hide_button(GWY_TOOL(tool), TRUE);
+    gwy_radio_buttons_set_current(tool->style, tool->args.style);
 
     g_object_unref(sizegroup);
     g_object_unref(tool->sensgroup);
@@ -506,7 +655,7 @@ gwy_tool_mask_editor_data_switched(GwyTool *gwytool,
         return;
 
     tool = GWY_TOOL_MASK_EDITOR(gwytool);
-    gwy_tool_mask_editor_setup_layer(tool);
+    gwy_tool_mask_editor_style_changed(tool);
     gwy_sensitivity_group_set_state(tool->sensgroup, SENS_DATA,
                                     data_view ? SENS_DATA : 0);
     gwy_tool_mask_editor_mask_changed(plain_tool);
@@ -530,10 +679,29 @@ gwy_tool_mask_editor_mask_changed(GwyPlainTool *plain_tool)
     gwy_sensitivity_group_set_state(tool->sensgroup, SENS_MASK, state);
 }
 
+static void
+gwy_tool_mask_editor_style_changed(GwyToolMaskEditor *tool)
+{
+    MaskEditStyle style;
+
+    style = tool->args.style = gwy_radio_buttons_get_current(tool->style);
+    if (style == MASK_EDIT_STYLE_SHAPES) {
+        /* Force layer setup */
+        tool->args.shape = -1;
+        gwy_tool_mask_editor_shape_changed(tool);
+    }
+    else {
+        gwy_plain_tool_connect_selection(GWY_PLAIN_TOOL(tool),
+                                         tool->layer_type_point, "pointer");
+        gwy_tool_mask_editor_setup_layer(tool);
+    }
+}
+
 static void
 gwy_tool_mask_editor_mode_changed(GwyToolMaskEditor *tool)
 {
     tool->args.mode = gwy_radio_buttons_get_current(tool->mode);
+    gwy_radio_buttons_set_current(tool->style, MASK_EDIT_STYLE_SHAPES);
     if (tool->args.mode == -1)
         g_warning("Mode set to -1!");
 }
@@ -544,10 +712,8 @@ gwy_tool_mask_editor_shape_changed(GwyToolMaskEditor *tool)
     MaskEditShape shape;
 
     shape = gwy_radio_buttons_get_current(tool->shape);
-    if (shape == tool->args.shape)
-        return;
     tool->args.shape = shape;
-
+    gwy_radio_buttons_set_current(tool->style, MASK_EDIT_STYLE_SHAPES);
     gwy_plain_tool_connect_selection(GWY_PLAIN_TOOL(tool),
                                      tool->layer_types[tool->args.shape],
                                      shape_selection_names[tool->args.shape]);
@@ -563,19 +729,64 @@ gwy_tool_mask_editor_setup_layer(GwyToolMaskEditor *tool)
     if (!plain_tool->data_view)
         return;
 
-    gwy_object_set_or_reset(plain_tool->layer,
-                            tool->layer_types[tool->args.shape],
-                            "editable", TRUE,
-                            "focus", -1,
-                            NULL);
-    if (tool->args.shape == MASK_SHAPE_LINE)
-        g_object_set(plain_tool->layer,
-                     "line-numbers", FALSE,
-                     "thickness", 1,
-                     NULL);
+    if (tool->args.style == MASK_EDIT_STYLE_SHAPES) {
+        gwy_object_set_or_reset(plain_tool->layer,
+                                tool->layer_types[tool->args.shape],
+                                "editable", TRUE,
+                                "focus", -1,
+                                NULL);
+        if (tool->args.shape == MASK_SHAPE_LINE)
+            g_object_set(plain_tool->layer,
+                         "line-numbers", FALSE,
+                         "thickness", 1,
+                         NULL);
+    }
+    else {
+        gwy_object_set_or_reset(plain_tool->layer, tool->layer_type_point,
+                                "editable", TRUE,
+                                "focus", -1,
+                                NULL);
+        if (tool->args.tool == MASK_TOOL_PAINT_DRAW
+            || tool->args.tool == MASK_TOOL_PAINT_ERASE)
+            g_object_set(plain_tool->layer,
+                         "marker-radius", tool->args.radius,
+                         NULL);
+        else
+            g_object_set(plain_tool->layer,
+                         "draw-marker", FALSE,
+                         NULL);
+    }
+
     gwy_selection_set_max_objects(plain_tool->selection, 1);
 }
 
+static void
+gwy_tool_mask_editor_tool_changed(GwyToolMaskEditor *tool)
+{
+    tool->args.tool = gwy_radio_buttons_get_current(tool->tool);
+    gwy_radio_buttons_set_current(tool->style, MASK_EDIT_STYLE_DRAWING);
+    gwy_tool_mask_editor_setup_layer(tool);
+}
+
+static void
+gwy_tool_mask_editor_radius_changed(GtkAdjustment *adj,
+                                    GwyToolMaskEditor *tool)
+{
+    tool->args.radius = gwy_adjustment_get_int(adj);
+    gwy_radio_buttons_set_current(tool->style, MASK_EDIT_STYLE_DRAWING);
+    if (tool->args.style == MASK_EDIT_STYLE_DRAWING
+        && (tool->args.tool == MASK_TOOL_PAINT_DRAW
+            || tool->args.tool == MASK_TOOL_PAINT_ERASE)) {
+        GwyPlainTool *plain_tool = GWY_PLAIN_TOOL(tool);
+
+        if (plain_tool->data_view && plain_tool->layer) {
+            g_object_set(plain_tool->layer,
+                         "marker-radius", tool->args.radius,
+                         NULL);
+        }
+    }
+}
+
 static void
 gwy_tool_mask_editor_gsamount_changed(GtkAdjustment *adj,
                                       GwyToolMaskEditor *tool)
@@ -1038,11 +1249,25 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
     g_return_if_fail(plain_tool->data_field);
 
     tool = GWY_TOOL_MASK_EDITOR(plain_tool);
+    tool->drawing_started = FALSE;
     if (!gwy_selection_get_object(plain_tool->selection, 0, sel))
         return;
 
     isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
     isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
+
+    if (tool->args.style == MASK_EDIT_STYLE_DRAWING) {
+        if (tool->args.tool == MASK_TOOL_PAINT_DRAW
+            || tool->args.tool == MASK_TOOL_PAINT_ERASE) {
+            /* The mask has been already modified. */
+            gwy_selection_clear(plain_tool->selection);
+            return;
+        }
+        gwy_tool_mask_editor_bucket_fill(tool, isel[0], isel[1]);
+        gwy_data_field_data_changed(plain_tool->mask_field);
+        return;
+    }
+
     isel[2] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[2]));
     isel[3] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[3]));
     if (tool->args.shape == MASK_SHAPE_LINE) {
@@ -1173,5 +1398,170 @@ gwy_tool_mask_editor_selection_finished(GwyPlainTool *plain_tool)
         gwy_data_field_data_changed(mfield);
 }
 
+static void
+gwy_tool_mask_editor_bucket_fill(GwyToolMaskEditor *tool,
+                                 gint j, gint i)
+{
+    GwyPlainTool *plain_tool;
+    GwyDataField *mfield;
+    gint xres, yres, k, gno;
+    gint *g, *grains = NULL;
+    gboolean draw;
+    gdouble *data;
+    GQuark quark;
+
+    plain_tool = GWY_PLAIN_TOOL(tool);
+    mfield = plain_tool->mask_field;
+    xres = gwy_data_field_get_xres(mfield);
+    yres = gwy_data_field_get_yres(mfield);
+    if (i < 0 || i >= yres || j < 0 || j >= xres)
+        return;
+
+    if (tool->args.tool == MASK_TOOL_FILL_DRAW)
+        draw = TRUE;
+    else if (tool->args.tool == MASK_TOOL_FILL_ERASE)
+        draw = FALSE;
+    else {
+        g_return_if_reached();
+    }
+
+    data = gwy_data_field_get_data(mfield);
+    if ((data[i*xres + j] && draw) || (!data[i*xres + j] && !draw))
+        return;
+
+    quark = gwy_app_get_mask_key_for_id(plain_tool->id);
+    gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
+
+    g = grains = g_new0(gint, xres*yres);
+    if (draw) {
+        gwy_data_field_multiply(mfield, -1.0);
+        gwy_data_field_add(mfield, 1.0);
+    }
+    gwy_data_field_number_grains(mfield, grains);
+    gno = grains[i*xres + j];
+
+    for (k = xres*yres; k; k--, data++, g++) {
+        if (*g == gno)
+            *data = 0.0;
+    }
+    if (draw) {
+        gwy_data_field_multiply(mfield, -1.0);
+        gwy_data_field_add(mfield, 1.0);
+    }
+
+    g_free(grains);
+}
+
+/* FIXME: This is woefully inefficient. */
+static void
+gwy_data_field_paint_wide_line(GwyDataField *dfield,
+                               gint col, gint row,
+                               gint width, gint height,
+                               gdouble radius, gdouble value)
+{
+    gint i, q, xres;
+
+    xres = gwy_data_field_get_xres(dfield);
+    if (ABS(height) >= width) {
+        q = width/2;
+        if (height > 0) {
+            for (i = 0; i <= height; i++) {
+                gwy_data_field_circular_area_fill(dfield,
+                                                  col + q/height, row + i,
+                                                  radius, value);
+                q += width;
+            }
+        }
+        else {
+            height = ABS(height);
+            for (i = 0; i <= height; i++) {
+                gwy_data_field_circular_area_fill(dfield,
+                                                  col + q/height, row - i,
+                                                  radius, value);
+                q += width;
+            }
+        }
+    }
+    else {
+        q = height/2;
+        for (i = 0; i <= width; i++) {
+            gwy_data_field_circular_area_fill(dfield,
+                                              col+i, row + q/width,
+                                              radius, value);
+            q += height;
+        }
+    }
+}
+
+static void
+gwy_tool_mask_editor_selection_changed(GwyPlainTool *plain_tool,
+                                       G_GNUC_UNUSED gint hint)
+{
+    GwyDataField *mfield;
+    GwyToolMaskEditor *tool;
+    GQuark quark;
+    gint xres, yres;
+    gdouble *data;
+    gdouble sel[2];
+    gdouble fillvalue, r;
+    gint isel[2];
+
+    tool = GWY_TOOL_MASK_EDITOR(plain_tool);
+    if (tool->args.style != MASK_EDIT_STYLE_DRAWING)
+        return;
+
+    if (tool->args.tool == MASK_TOOL_PAINT_DRAW)
+        fillvalue = 1.0;
+    else if (tool->args.tool == MASK_TOOL_PAINT_ERASE)
+        fillvalue = 0.0;
+    else
+        return;
+
+    /* Apparently this gets called also during the tool destruction. */
+    if (!plain_tool->data_field || !plain_tool->selection
+        || !gwy_selection_get_object(plain_tool->selection, 0, sel)) {
+        tool->drawing_started = FALSE;
+        return;
+    }
+
+    isel[0] = floor(gwy_data_field_rtoj(plain_tool->data_field, sel[0]));
+    isel[1] = floor(gwy_data_field_rtoi(plain_tool->data_field, sel[1]));
+
+    quark = gwy_app_get_mask_key_for_id(plain_tool->id);
+
+    mfield = gwy_tool_mask_editor_maybe_add_mask(plain_tool, quark);
+    xres = gwy_data_field_get_xres(mfield);
+    yres = gwy_data_field_get_yres(mfield);
+    data = gwy_data_field_get_data(mfield);
+    r = tool->args.radius - 0.5;
+    if (isel[0] >= 0 && isel[0] < xres && isel[1] >= 0 && isel[1] < yres) {
+        if (!tool->drawing_started) {
+            gwy_app_undo_qcheckpointv(plain_tool->container, 1, &quark);
+            gwy_data_field_circular_area_fill(mfield, isel[0], isel[1],
+                                              r, fillvalue);
+        }
+        else {
+            gint xy[4];
+
+            xy[0] = tool->oldisel[0];
+            xy[1] = tool->oldisel[1];
+            xy[2] = isel[0];
+            xy[3] = isel[1];
+            if (xy[2] < xy[0]) {
+                GWY_SWAP(gdouble, xy[0], xy[2]);
+                GWY_SWAP(gdouble, xy[1], xy[3]);
+            }
+            xy[2] -= xy[0] - 1;
+            xy[3] -= xy[1] - 1;
+            gwy_data_field_paint_wide_line(mfield, xy[0], xy[1], xy[2], xy[3],
+                                           r, fillvalue);
+        }
+        gwy_data_field_data_changed(mfield);
+        tool->oldisel[0] = isel[0];
+        tool->oldisel[1] = isel[1];
+        tool->drawing_started = TRUE;
+    }
+}
+
 /* vim: set cin et ts=4 sw=4 cino=>1s,e0,n0,f0,{0,}0,^0,\:1s,=0,g1s,h0,t0,+1s,c3,(0,u0 : */
 
