@@ -1,5 +1,5 @@
 /*
- *  @(#) $Id: spectro.c 8342 2007-07-12 21:19:05Z yeti-dn $
+ *  @(#) $Id: spectro.c 11600 2010-11-23 09:39:42Z yeti-dn $
  *  Copyright (C) 2003-2007 Owain Davies, David Necas (Yeti), Petr Klapetek.
  *  E-mail: owain.davies@blueyonder.co.uk, yeti@gwyddion.net,
  *          klapetek@gwyddion.net.
@@ -240,7 +240,7 @@ gwy_tool_spectro_init_dialog(GwyToolSpectro *tool)
     GtkDialog *dialog;
     GtkWidget *scwin, *label, *hbox, *vbox, *hbox2;
     GtkTable *table;
-    GtkTreeSelection *select;
+    GtkTreeSelection *selection;
     guint i, row;
 
     dialog = GTK_DIALOG(GWY_TOOL(tool)->dialog);
@@ -274,9 +274,9 @@ gwy_tool_spectro_init_dialog(GwyToolSpectro *tool)
         gtk_tree_view_append_column(tool->treeview, column);
     }
 
-    select = gtk_tree_view_get_selection(GTK_TREE_VIEW (tool->treeview));
-    gtk_tree_selection_set_mode(select, GTK_SELECTION_MULTIPLE);
-    g_signal_connect(G_OBJECT(select), "changed",
+    selection = gtk_tree_view_get_selection(GTK_TREE_VIEW (tool->treeview));
+    gtk_tree_selection_set_mode(selection, GTK_SELECTION_MULTIPLE);
+    g_signal_connect(G_OBJECT(selection), "changed",
                      G_CALLBACK(gwy_tool_spectro_tree_sel_changed),
                      tool);
 
